@@ -116,6 +116,7 @@ const SEND_MESSAGE_SIG = "sendMessage(java.util.Map,int,java.lang.String,boolean
 const DELETE_ADDRESS_SIG = "deleteAddress(java.lang.String)";
 const DELETE_MESSAGE_SIG = "removeMessage(long)";
 const MOVE_MESSAGE_SIG = "moveMessage(long,java.lang.String)";
+const COPY_MESSAGE_SIG = "copyMessage(long,java.lang.String)";
 const CREATE_QUEUE_SIG = "createQueue(java.lang.String,boolean)"
 const CREATE_ADDRESS_SIG = "createAddress(java.lang.String,java.lang.String)"
 const COUNT_MESSAGES_SIG = "countMessages()";
@@ -143,9 +144,11 @@ const typeLabels = ["DEFAULT", "1", "object", "text", "bytes", "map", "stream",
 class ArtemisService {
 
     private brokerObjectName: Promise<string>
+    private brokerInfo: Promise<BrokerInfo>
 
     constructor() {
         this.brokerObjectName = this.initBrokerObjectName();
+        this.brokerInfo = this.initBrokerInfo();
     }
 
     private async initBrokerObjectName(): Promise<string> {
@@ -156,7 +159,7 @@ class ArtemisService {
 
 
 
-    async createBrokerInfo(): Promise<BrokerInfo> {
+    async initBrokerInfo(): Promise<BrokerInfo> {
         return new Promise<BrokerInfo>(async (resolve, reject) => {
             var brokerObjectName = await this.brokerObjectName;
             var response = await jolokiaService.readAttributes(brokerObjectName);
@@ -195,10 +198,17 @@ class ArtemisService {
         });
     }
 
+    async getBrokerInfo(): Promise<BrokerInfo> {
+        return await this.brokerInfo;
+    }
+
     async createBrokerTopology(): Promise<BrokerTopology> {
         return new Promise<BrokerTopology>(async (resolve, reject) => {
             try {
-                var brokerInfo = await this.createBrokerInfo();
+                var brokerInfo = await this.getBrokerInfo();
+                var brokerObjectName = await this.brokerObjectName;
+                const topology = await jolokiaService.execute(brokerObjectName, LIST_NETWORK_TOPOLOGY_SIG) as string;
+                brokerInfo.networkTopology =  new BrokerNetworkTopology(JSON.parse(topology));
                 var brokerTopology: BrokerTopology = {
                     broker: brokerInfo,
                     addresses: []
@@ -302,6 +312,11 @@ class ArtemisService {
         return jolokiaService.execute(mbean, MOVE_MESSAGE_SIG, [id, targetQueue])
     }
 
+    async copyMessage(id: number, targetQueue: string,  address: string, routingType: string, queue: string) {
+        const mbean = createQueueObjectName(await this.getBrokerObjectName(), address, routingType, queue);
+        return jolokiaService.execute(mbean, COPY_MESSAGE_SIG, [id, targetQueue])
+    }
+
 
     async createQueue(queueConfiguration: string) {
         return await jolokiaService.execute(await this.getBrokerObjectName(), CREATE_QUEUE_SIG, [queueConfiguration, false]).then().catch() as string;
@@ -589,6 +604,20 @@ class ArtemisService {
     checkCanBrowseQueue = (queueMBean: MBeanNode ): boolean => {
         return (this.DEBUG_PRIVS && queueMBean?.hasInvokeRights(BROWSE_SIG)) ?? false;
     }
+
+    doesCopyMessageMethodExist = (broker: MBeanNode | undefined, queue: string): boolean => {
+        return this.doesMethodExist(broker, queue, COPY_MESSAGE_SIG);
+    }
+
+    doesMethodExist = (broker: MBeanNode | undefined, queue: string, method: string): boolean => {
+        if(broker) {
+            var queueMBean = broker.parent?.find(node => { 
+                return node.propertyList?.get('subcomponent') === 'queues' && node.name === queue 
+            })
+            return queueMBean?queueMBean.hasOperations(method): false;
+        }
+        return false;
+    }
 }
 
 export const artemisService = new ArtemisService()
\ No newline at end of file
