@@ -14,11 +14,11 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-import React, { useEffect, useState } from 'react'
+import React, { useContext, useEffect, useState } from 'react'
 import { Column } from '../table/ArtemisTable';
 import { artemisService } from '../artemis-service';
 import { Toolbar, ToolbarContent, ToolbarItem, Text, SearchInput, Button, PaginationVariant, Pagination, DataList, DataListCell, DataListCheck, DataListItem, DataListItemCells, DataListItemRow, Modal, TextContent, Icon, ModalVariant } from '@patternfly/react-core';
-import { TableComposable, Thead, Tr, Th, Tbody, Td, ActionsColumn, IAction } from '@patternfly/react-table';
+import { Thead, Tr, Th, Tbody, Td, ActionsColumn, IAction, Table } from '@patternfly/react-table';
 import ExclamationCircleIcon from '@patternfly/react-icons/dist/esm/icons/exclamation-circle-icon';
 import { createQueueObjectName } from '../util/jmx';
 import { Link } from 'react-router-dom';
@@ -26,6 +26,9 @@ import { eventService } from '@hawtio/react';
 import { QueueSelectInput } from './QueueSelect';
 import { SendMessage } from './SendMessage';
 import { Message } from './MessageView';
+import { ArtemisContext } from '../context';
+import { log } from '../globals';
+import { artemisPreferencesService, columnStorage } from '../artemis-preferences-service';
 
 export type MessageProps = {
   address: string,
@@ -64,15 +67,19 @@ export const MessagesTable: React.FunctionComponent<MessageProps> = props => {
   const [rows, setRows] = useState([])
   const [perPage, setPerPage] = useState(10);
   const [columns, setColumns] = useState(allColumns);
+  const [columnsLoaded, setColumnsLoaded] = useState(false);
   const [columnsModalOpen, setColumnsModalOpen] = useState(false);
   const [resultsSize, setresultsSize] = useState(0);
   const [selectedMessages, setSelectedMessages] = useState<number[]>([]);
   const [selectedTargetQueue, setSelectedTargetQueue] = useState<string>('');
   const [showDeleteMessagesModal, setShowDeleteMessagesModal] = useState(false);
   const [showMoveMessagesModal, setShowMoveMessagesModal] = useState(false);
+  const [showCopyMessagesModal, setShowCopyMessagesModal] = useState(false);
   const [showResendModal, setShowResendModal] = useState(false);
   const [ resendMessage, setResendMessage] = useState<Message | undefined>();
 
+  const { brokerNode } = useContext(ArtemisContext);
+
   useEffect(() => {
     const listData = async () => {
       var data = await listMessages();
@@ -85,6 +92,12 @@ export const MessagesTable: React.FunctionComponent<MessageProps> = props => {
       const response = await artemisService.getMessages(queueMBean, page, perPage, filter);
       return response;
     }
+    setPerPage(artemisPreferencesService.loadTablePageSize(columnStorage.messages));
+    if (!columnsLoaded) {
+      const updatedColumns: Column[] = artemisPreferencesService.loadColumnPreferences(columnStorage.messages, allColumns);
+      setColumns(updatedColumns);
+      setColumnsLoaded(true);
+    }
     listData();
 
   }, [props.address, props.routingType, props.queue, page, perPage, filter, selectedMessages])
@@ -94,7 +107,9 @@ export const MessagesTable: React.FunctionComponent<MessageProps> = props => {
   };
 
   const handlePerPageSelect = (_event: React.MouseEvent | React.KeyboardEvent | MouseEvent, newPerPage: number, newPage: number) => {
+    artemisPreferencesService.saveTablePageSize(columnStorage.messages, newPerPage)
     setPerPage(newPerPage);
+    selectAllMessages(false);
   };
 
   const handleColumnsModalToggle = () => {
@@ -150,6 +165,7 @@ export const MessagesTable: React.FunctionComponent<MessageProps> = props => {
 
   const onSave = () => {
     setColumnsModalOpen(!columnsModalOpen);
+    artemisPreferencesService.saveColumnPreferences(columnStorage.messages, columns);
   };
 
   const updateColumnStatus = (index: number, column: Column) => {
@@ -176,6 +192,13 @@ export const MessagesTable: React.FunctionComponent<MessageProps> = props => {
     return selectedMessages.includes(id);
   }
 
+  const doesCopyMessagemethodExist = artemisService.doesMethodExist(brokerNode, props.queue, "copyMessage");
+
+  if(!doesCopyMessagemethodExist) {
+    log.warn("Copy button method not available in this version of Artemis");
+    
+  }
+
   const selectAllMessages = (isSelecting: boolean) => {
     if(isSelecting) {
         var updatedSelectedMessages: number[] = rows.map((row: any, index) => {
@@ -204,12 +227,12 @@ export const MessagesTable: React.FunctionComponent<MessageProps> = props => {
         if (rejectedReasons.length > 0) {
           eventService.notify({
             type: 'warning',
-            message: "not all messages deleted: errors " + rejectedReasons.toString(),
+            message: "Not all messages deleted: errors " + rejectedReasons.toString(),
           })
         } else {
           eventService.notify({
             type: 'success',
-            message: "Messages Successfully Deleted [" + selectedMessages + "]",
+            message: "Messages deleted successfully [" + selectedMessages + "]",
           })
         }
       });
@@ -233,12 +256,12 @@ export const MessagesTable: React.FunctionComponent<MessageProps> = props => {
         if (rejectedReasons.length > 0) {
           eventService.notify({
             type: 'warning',
-            message: "not all messages moved: errors " + rejectedReasons.toString(),
+            message: "Not all messages moved: errors " + rejectedReasons.toString(),
           })
         } else {
           eventService.notify({
             type: 'success',
-            message: "Messages Successfully Moved [" + selectedMessages + "]",
+            message: "Messages moved successfully [" + selectedMessages + "]",
           })
         }
       });
@@ -247,6 +270,35 @@ export const MessagesTable: React.FunctionComponent<MessageProps> = props => {
     setSelectedMessages([]);
   }
 
+  const handleCopyMessages = () => {
+
+    const isRejected = <T,>(p: PromiseSettledResult<T>): p is PromiseRejectedResult => p.status === 'rejected';
+    var results: Promise<unknown>[] = [];
+    for (let i = 0; i < selectedMessages.length; i++) {
+      var promise: Promise<unknown> = artemisService.copyMessage(selectedMessages[i], selectedTargetQueue, props.address, props.routingType, props.queue);
+      results.push(promise);
+    };
+    Promise.allSettled(results)
+      .then((results) => {
+        const rejectedReasons = results.filter(isRejected).map(p => p.reason);
+
+        if (rejectedReasons.length > 0) {
+          eventService.notify({
+            type: 'warning',
+            message: "not all messages copied: errors " + rejectedReasons.toString(),
+          })
+        } else {
+          eventService.notify({
+            type: 'success',
+            message: "Messages Successfully Copied [" + selectedMessages + "]",
+          })
+        }
+      });
+
+    setShowCopyMessagesModal(false);
+    setSelectedMessages([]);
+  }
+
   return (
     <React.Fragment>
       <Toolbar id="toolbar">
@@ -272,11 +324,14 @@ export const MessagesTable: React.FunctionComponent<MessageProps> = props => {
             <Button onClick={() => setShowMoveMessagesModal(true)}>Move</Button>
           </ToolbarItem>
           <ToolbarItem>
-            <Button variant='link' onClick={handleColumnsModalToggle}>Manage Columns</Button>
+            <Button onClick={() => setShowCopyMessagesModal(true)} isDisabled={!doesCopyMessagemethodExist} >Copy</Button>
+          </ToolbarItem>
+          <ToolbarItem>
+            <Button variant='link' onClick={handleColumnsModalToggle} >Manage Columns</Button>
           </ToolbarItem>
         </ToolbarContent>
       </Toolbar>
-      <TableComposable id='message-table' variant="compact" aria-label="Column Management Table">
+      <Table id='message-table' variant="compact" aria-label="Column Management Table">
         <Thead>
           <Tr >
             <Th
@@ -324,7 +379,7 @@ export const MessagesTable: React.FunctionComponent<MessageProps> = props => {
             </Tr>
           ))}
         </Tbody>
-      </TableComposable>
+      </Table>
       <Pagination
         itemCount={resultsSize}
         page={page}
@@ -333,7 +388,7 @@ export const MessagesTable: React.FunctionComponent<MessageProps> = props => {
         onPerPageSelect={handlePerPageSelect}
         variant={PaginationVariant.bottom}
         titles={{
-          paginationTitle: `${PaginationVariant.bottom} pagination`
+          paginationAriaLabel: `${PaginationVariant.bottom} pagination`
         }}
       />
       {props.back &&
@@ -363,7 +418,7 @@ export const MessagesTable: React.FunctionComponent<MessageProps> = props => {
             You are about to delete message(s) {selectedMessages.toString()}
           </Text>
           <Text component="p">
-            This operation cannot be undone so please be careful.
+            This operation cannot be undone.
           </Text>
         </TextContent>
       </Modal>
@@ -389,6 +444,34 @@ export const MessagesTable: React.FunctionComponent<MessageProps> = props => {
             </Icon>
             You are about to move messages {selectedMessages.toString()}
           </Text>
+          <Text component="p">
+            This operation cannot be undone.
+          </Text>
+          <QueueSelectInput selectQueue={setSelectedTargetQueue}/>
+        </TextContent>
+      </Modal>
+      <Modal
+        aria-label='copy-message-modal'
+        variant={ModalVariant.medium}
+        isOpen={showCopyMessagesModal}
+        actions={[
+          <Button key="cancel" variant="secondary" onClick={() => setShowCopyMessagesModal(false)}>
+            Cancel
+          </Button>,
+          <Button key="copy" variant="primary" onClick={handleCopyMessages}>
+            Confirm
+          </Button>
+        ]}>
+        <TextContent>
+          <Text component="h2">
+            Confirm Copy Message(s)
+          </Text>
+          <Text component="p">
+            <Icon isInline status='warning'>
+              <ExclamationCircleIcon />
+            </Icon>
+            You are about to copy messages {selectedMessages.toString()}
+          </Text>
           <Text component="p">
             This operation cannot be undone so please be careful.
           </Text>
@@ -414,7 +497,7 @@ export const MessagesTable: React.FunctionComponent<MessageProps> = props => {
         variant="small"
         description={
           <TextContent>
-            <Text>Selected categories will be displayed in the table.</Text>
+            <Text>Selected categories are displayed in the table.</Text>
             <Button isInline onClick={selectAllColumns} variant="link">
               Select all
             </Button>
@@ -455,4 +538,4 @@ export const MessagesTable: React.FunctionComponent<MessageProps> = props => {
       </Modal>
     </React.Fragment>
   );
-}
\ No newline at end of file
+}
